@@ -4,16 +4,11 @@ package Net::ISC::DHCPd;
 
 Net::ISC::DHCPd - Interacts with ISC DHCPd
 
-=head1 VERSION
-
-0.05_02
-
 =head1 SYNOPSIS
 
  my $dhcpd = Net::ISC::DHCPd->new(
-                 config => { file => "path/to/config" },
-                 leases => { file => "path/to/leases" },
-                 omapi => { ... },
+                 -config => { file => "path/to/config" },
+                 -leases => { file => "path/to/leases" },
              );
 
 See tests for more documentation.
@@ -21,74 +16,53 @@ See tests for more documentation.
 =cut
 
 use Moose;
-use Moose::Util::TypeConstraints;
-use Net::ISC::DHCPd::Types ':all';
 use File::Basename;
 use File::Path;
 use File::Temp;
-use Net::ISC::DHCPd::Process;
 
-our $VERSION = '0.05_02';
+our $VERSION = "0.01";
 
 =head1 OBJECT ATTRIBUTES
 
 =head2 config
 
  $config_obj = $self->config
- $bool = $self->has_config;
 
-Instance of L<Net::ISC::DHCPd::Config> class.
+Default: L<Net::ISC::DHCPd::Config>.
 
 =cut
 
 has config => (
     is => 'ro',
-    isa => ConfigObject,
-    coerce => 1,
-    lazy_build => 1,
+    lazy => 1,
+    isa => 'Net::ISC::DHCPd::Config',
+    default => sub {
+        eval "require Net::ISC::DHCPd::Config" or confess $@;
+        Net::ISC::DHCPd::Config->new;
+    },
 );
 
-*_build_config = sub { _build_child_obj(Config => @_) };
-
 =head2 leases
 
  $leases_obj = $self->leases
- $bool = $self->has_leases;
 
-Instance of L<Net::ISC::DHCPd::Leases> class.
+Default: L<Net::ISC::DHCPd::Leases>.
 
 =cut
 
 has leases => (
     is => 'ro',
-    isa => LeasesObject,
-    coerce => 1,
-    lazy_build => 1,
+    lazy => 1,
+    isa => 'Net::ISC::DHCPd::Leases',
+    default => sub {
+        eval "require Net::ISC::DHCPd::Leases" or confess $@;
+        Net::ISC::DHCPd::Leases->new;
+    },
 );
 
-*_build_leases = sub { _build_child_obj(Leases => @_) };
-
-=head2 omapi
-
- $omapi_obj = $self->omapi;
- $bool = $self->has_omapi;
-
-Instance of L<Net::ISC::DHCPd::OMAPI> class.
-
-=cut
-
-has omapi => (
-    is => 'ro',
-    isa => OMAPIObject,
-    coerce => 1,
-    lazy_build => 1,
-);
-
-*_build_omapi = sub { _build_child_obj(OMAPI => @_) };
-
 =head2 binary
 
- $path_to_binary = $self->binary;
+ $path_to_binary = $self->binary
 
 Default: "dhcpd3"
 
@@ -102,7 +76,8 @@ has binary => (
 
 =head2 pidfile
 
- $path_to_pidfile = $self->pidfile;
+ $self->pidfile($path_to_pidfile)
+ $path_to_pidfile = $self->pidfile
 
 Default: /var/run/dhcp3-server/dhcpd.pid
 
@@ -116,11 +91,7 @@ has pidfile => (
 
 =head2 process
 
- $proc_obj = $self->process;
- $self->process($proc_obj);
- $self->process(\%args);
- $self->has_process;
- $self->clear_process;
+ $proc_obj = $self->process
 
 The object holding the dhcpd process.
 
@@ -128,20 +99,28 @@ The object holding the dhcpd process.
 
 has process => (
     is => 'rw',
-    isa => ProcessObject,
-    coerce => 1,
-    lazy_build => 1,
 );
 
-sub _build_process {
-    confess 'process() cannot be build. Usage: $self->process($process_obj)';
-}
+=head2 process_class
+
+ $classname = $self->process_class
+
+The class that should spawn the L<process> object. Needs to support
+the minimum api of L<Net::ISC::DHCPd::Process>.
+
+=cut
+
+has process_class => (
+    is => 'ro',
+    isa => 'Str',
+    default => 'Net::ISC::DHCPd::Process',
+);
 
 =head2 errstr
 
- $string = $self->errstr;
+ $string = $self->errstr
 
-Holds the last know error.
+Holds the last know errorr.
 
 =cut
 
@@ -153,9 +132,44 @@ has errstr => (
 
 =head1 METHODS
 
+=head2 BUILD
+
+Takes extra '-foo' arguments to C<new()> and appends the values to
+the appropriate attribute.
+
+Example:
+
+ $self = $class->new(
+   -config => {
+     file => "/foo/bar.conf",
+   },
+ );
+
+Will result in:
+
+ $self = $class->new;
+ $self->config->file("/foo/bar.conf");
+
+=cut
+
+sub BUILD {
+    my $self = shift;
+    my $args = shift;
+
+    for my $attr (%$args) {
+        next unless(ref $args->{$attr} eq 'HASH');
+        next unless($attr =~ s/^-//);
+        next unless($self->has_attribute($attr));
+
+        for my $key (%$attr) {
+            $self->$attr->$key($attr->{$key});
+        }
+    }
+}
+
 =head2 start
 
- $bool = $self->start($args);
+ $bool = $self->start($args)
 
 Will start the dhcpd server, as long as there is no existing process.
 
@@ -180,9 +194,10 @@ TODO: Enable it to start the server as a differnet user/group.
 sub start {
     my $self = shift;
     my $args = shift || {};
+    my $proc = $self->process;
     my($user, $group);
 
-    if($self->has_process and $self->process->kill(0)) {
+    if($proc and $proc->kill(0)) {
         $self->errstr('allready running');
         return 0;
     }
@@ -205,35 +220,31 @@ sub start {
     for my $file ($self->config->file, $self->leases->file, $self->pidfile) {
         my $dir = dirname $file;
         next if -d $dir;
+        eval { File::Path::mkpath($dir) };
 
-        eval {
-            File::Path::mkpath($dir);
-            1;
-        } or do {
+        if($@) {
             $self->errstr("could not mkpath($dir): $@");
             return;
-        };
-
+        }
         unless(chown $user, $group, $dir) {
             $self->errstr("could not chown($user, $group $dir): $!");
             return;
         }
     }
 
-    $self->process({
+    $self->process($self->process_class->new(
         program => $self->binary,
         args    => $args,
         user    => $user,
         group   => $group,
-        start   => 1,
-    });
+    ));
 
     return $self->process ? 1 : undef;
 }
 
 =head2 stop
 
- $bool = $self->stop;
+ $bool = $self->stop
 
 Return:
 
@@ -244,13 +255,14 @@ Return:
 
 sub stop {
     my $self = shift;
+    my $proc = $self->process;
 
-    unless($self->has_process) {
+    unless($proc) {
         $self->errstr("no such process");
         return undef;
     }
 
-    unless($self->process->kill('TERM')) {
+    unless($proc->kill('TERM')) {
         $self->errstr("Could not send signal to process");
         return undef;
     }
@@ -273,7 +285,11 @@ sub restart {
     my $self = shift;
     my $proc;
     
-    if($self->has_process and !$self->stop) {
+    unless($proc = $self->process) {
+        $self->errstr("no such process");
+        return undef;
+    }
+    unless($self->stop) {
         return undef;
     }
     unless($self->start) {
@@ -285,7 +301,7 @@ sub restart {
 
 =head2 status
 
- $string = $self->status;
+ $string = $self->status
 
 Returns the status of the DHCPd server:
 
@@ -297,8 +313,8 @@ Returns the status of the DHCPd server:
 sub status {
     my $self = shift;
 
-    if($self->has_process) {
-        if($self->process->kill(0)) {
+    if(my $proc = $self->process) {
+        if($proc->kill(0)) {
             return "running";
         }
     }
@@ -308,8 +324,8 @@ sub status {
 
 =head2 test
 
- $bool = $self->test("config");
- $bool = $self->test("leases");
+ $bool = $self->test("config")
+ $bool = $self->test("leases")
 
 Will test either config or leases file.
 
@@ -321,90 +337,36 @@ Will test either config or leases file.
 sub test {
     my $self = shift;
     my $what = shift || q();
-    my $exit;
+    my $ret;
 
     if($what eq 'config') {
-        my $tmp = File::Temp->new;
-        print $tmp $self->config->generate;
-        $exit = $self->_run('-t', '-cf', $tmp->filename);
+        my $file = File::Temp->new;
+        print $file $self->config->generate;
+        system $self->binary, '-t', '-cf', $file->filename;
     }
     elsif($what eq 'leases') {
-        $exit = $self->_run('-t', '-lf', $self->leases->file);
+        system $self->binary, '-t', '-lf', $self->leases->file;
     }
     else {
         $self->errstr('Invalid argument');
         return;
     }
 
-    if($exit and $exit == -1) {
+    if($? == -1) {
         $self->errstr($!);
         return;
     }
-    elsif($exit) {
-        $self->errstr($exit >> 8);
+    else {
+        $self->errstr($? >> 8);
         return;
     }
 
     return 1;
 }
 
-sub _run {
-    my $self = shift;
-    my @args = @_;
-    my $exit;
-
-    local *OLDERR;
-    local *OLDOUT;
-    open OLDERR, ">&", \*STDERR;
-    open OLDOUT, ">&", \*STDOUT;
-    close STDERR;
-    close STDOUT;
-
-    $exit = system $self->binary, @args;
-
-    open STDERR, ">&", \*OLDERR;
-    open STDOUT, ">&", \*OLDOUT;
-
-    return $exit;
-}
-
-# used from attributes
-sub _build_child_obj {
-    my $type = shift;
-    my $self = shift;
-    my $args = shift || {};
-
-    eval "require Net::ISC::DHCPd::$type" or confess $@;
-
-    return "Net::ISC::DHCPd::$type"->new($args);
-}
-
-=head1 BUGS
-
-Please report any bugs or feature requests to
-C<bug-net-isc-dhcpd at rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Net-ISC-DHCPd>.
-I will be notified, and then you'll automatically be notified of progress on
-your bug as I make changes.
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2007 Jan Henning Thorsen, all rights reserved.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
 =head1 AUTHOR
 
-Jan Henning Thorsen, C<< <jhthorsen at cpan.org> >>
-
-=head1 CONTRIBUTORS
-
-Nito Martinez
-
-Alexey Illarionov
-
-Patrick
+Jan Henning Thorsen
 
 =cut
 
