@@ -74,10 +74,6 @@ __PACKAGE__->create_children(qw/
 
 =head2 file
 
- $str = $self->file;
-
-Holds the path to the config file.
-
 =cut
 
 has file => (
@@ -88,30 +84,20 @@ has file => (
 
 =head2 filehandle
 
- $glob = $self->filehandle;
- $bool = $self->has_filehandle;
- $self->clear_filehandle;
-
-Holds the filehande to l<file>.
-
 =cut
 
 has filehandle => (
-    is => 'ro',
-    lazy_build => 1,
+    is => 'rw',
+    default => sub {
+        my $self = shift;
+        my $file = $self->file or return;
+        open(my $FH, "<", $file) or return;
+        $self->filehandle($FH);
+    },
 );
 
-sub _build_filehandle {
-    my $self = shift;
-    my $file = $self->file or confess 'file attribute needs to be set';
-    open(my $FH, "<", $file) or confess "cannot open $file: $!";
-    return $FH;
-}
-
 =head2 root
 
- $self = $self->root;
-
 =cut
 
 has '+root' => (
@@ -120,13 +106,10 @@ has '+root' => (
 
 =head2 parent
 
- $self = $self->parent;
-
 =cut
 
 has '+parent' => (
-    isa => 'Undef',
-    default => sub { undef },
+    default => sub { shift },
 );
 
 =head2 subnets
@@ -143,6 +126,10 @@ List of parsed L<Net::ISC::DHCPd::Config::Option> objects.
 
 =head1 METHODS
 
+=head2 new
+
+ $obj = $self->new(...);
+
 =head2 generate
 
  $config_text = $self->generate;
